#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include "gen.h"
#include "bitv.h"
#include "set.h"
#include "graph_am.h"

extern int parse_udg_cap(FILE *fp, Graph *g, int var);
static void heurCapMDS(Graph *g, nodeSet *S);

/* GLOBAL VARIABLES */
static Graph           g;
static nodeSet         D;

int main(int argc, char **argv)
{
  FILE           *fp = NULL;
  struct timeval  t1, t2;
  unsigned long   numSecs;

  if (argc < 3) {
    fprintf(stderr, "Usage: heur-capmds <filename> <uni-cap(0)/var-cap(1)>\n");
    exit(1);
  }

  if ((fp = fopen(argv[1], "r")) == NULL) {
    fprintf(stderr, "Error opening file %s\n", argv[1]);
    exit(2);
  }

  initGraph(&g);
  parse_udg_cap(fp, &g, atoi(argv[2]));

/*
  printGraph(&g);
*/

  gettimeofday(&t1, NULL);
  heurCapMDS(&g, &D);
  gettimeofday(&t2, NULL);

  numSecs = t2.tv_sec - t1.tv_sec;

  if (D.size > 0) {
    printf("%s %d %ld\n", argv[1], D.size, numSecs);
/*
    printf("Minimum Capacitated Dominating Set Found is\n");
    printNodeSet(&D);
*/
  }

  exit(0);
}


static void heurCapMDS(Graph *g, nodeSet *S)
{
  int           i, maxNode = -1, maxCap = 0, cap;
  Node_t       *node;

  /* Initialize the color of all nodes to WHITE */
  initColorWhite(g);

  while (TRUE) {
    maxCap = 0;
    cap = 0;

    for (i = 0; i <= g->maxNodeId; i++) {
      if (!isValidNode(i, &(g->nodeTable[i])))
        continue;

      node = &(g->nodeTable[i]); 
      if (node->color != BLACK) {
        cap = calcWhiteDegCovbyCap(g, node);
/*
        printf("cap returned for %d = %d\n", g->nodeTable[i].nodeId, cap);
*/
        if (maxCap < cap) {
          maxCap = cap;
          maxNode = node->nodeId;
/*
          printf("Node with max white degree covered by capacity (%d) is %d\n",
                  cap, maxNode);
*/
        }
      }
    }
    /* If no node with white degree is found, then, no node is white */
    if (maxCap == 0) {
/*
      printf("No node with white degree is found\n");
*/
      break;
    }

    /* Add the maxNode to S and mark it black; mark all its neighbors GREY */
/*
    printf("Adding node %d with capacity %d to the DS\n", maxNode, maxCap);
*/
    if (g->nodeTable[maxNode].color == WHITE)
      g->nodeTable[maxNode].resCap--;
    g->nodeTable[maxNode].color = BLACK;
    g->nodeTable[maxNode].ref++;
    addNodetoNodeSet(maxNode, S);
    markAdjNodesGrey(g, maxNode, TRUE);
  }

  return;
}

