#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gen.h"
#include "bitv.h"
#include "popu_t.h"

/* Initialise the population related data items   */
void popuInit(population_t *pop)
{
  memset(pop->table, 0, sizeof(popu_member_t) * MAX_INIT_POPULATION);
	pop->totalMembers = 0;
}

popu_member_t *popuExistMember(population_t *pop, bitvec_t *bvec, 
                               float fitness)
{
  int                i;

  popu_member_t      *node = NULL;

  for (i = 0; i < pop->totalMembers; i++) {
		node = &(pop->table[i]);
    if (fitness == node->fitness) {
    	if (!memcmp((void *)bvec, (void *)&(node->vec), sizeof(bitvec_t)))
        return node;
    }
  }

	return NULL;
}

#if 0
/* Function that returns the member numbered 'index' in the population */
popu_member_t *popuSearchByIndex(population_t *pop, int index)
{
  popu_member_t   *member = pop->head;
  int             i;

  for (i = 0; i < index; i++) {
    member = member->next;
  }

  return member;
}
#endif /* 0 */

/* Function that adds the given bitvector if not a duplicate           */
int popuAdd(population_t *pop, bitvec_t *bvec, float fitness)
{
  popu_member_t       *node = NULL; 

/*
  popu_member_t       *temp = NULL;
	if ((temp = popuExistMember(pop, bvec, fitness)) != NULL) {
    fprintf(stderr, "Member is a duplicate\n");
    return -1;
  }
*/

  node = &(pop->table[pop->totalMembers]);
  memcpy((void *)&(node->vec), (void *)bvec, sizeof(bitvec_t));
  node->fitness = fitness;
  pop->totalMembers++;
/*
  printf("%s: added member %d with count %d and fitness %f to the population\n", 
          __FUNCTION__, pop->totalMembers, node->vec.count, node->fitness);
*/

  return 0;
}

int popuReplace(population_t *pop, bitvec_t *bvec, float fitness)
{
  int               i, tailIndex = -1;
  float             tailFitness = 0.0;
  popu_member_t    *node = NULL;

  for (i = 0; i < pop->totalMembers; i++) {
    if (pop->table[i].fitness > tailFitness) {
      tailFitness = pop->table[i].fitness;
      tailIndex = i;
    }
  }

  if (tailIndex == -1) {
    fprintf(stderr, "%s: Error, No members found in population\n", 
            __FUNCTION__);
    return -1;
  }

  node = &(pop->table[tailIndex]);
  memcpy((void *)&(node->vec), (void *)bvec, sizeof(bitvec_t));
  node->fitness = fitness;

  return 0;
}

#if 0
/* Function that deletes the 'member' of the population               */
void popuDelete(population_t *pop, popu_member_t *member)
{
  if (member == NULL)
    return;

  if (pop->totalMembers == 1) {
    pop->head = pop->tail = NULL;
  } else if (pop->head == member) {
    pop->head = pop->head->next;
	} else if (member->next == NULL) {
    member->prev->next = NULL;
    pop->tail = member->prev;
	} else {
    member->prev->next = member->next;
		member->next->prev = member->prev;
	}

  pop->totalMembers--;
  free(member);
  return;
}

/* Function that deletes the tail or the worst member of the population  */
void popuDeleteTail(population_t *pop)
{
  popu_member_t      *node = pop->tail;

  if (pop->head == pop->tail) {
    pop->head = pop->tail = NULL;
  } else { 
    pop->tail->prev->next = NULL;
    pop->tail = pop->tail->prev;
  }

  pop->totalMembers--;
  free(node);
  return;
}

/* Function that deletes the member numbered 'nodeNum'                  */
void popuDeleteNumber(population_t *pop, int nodeNum)
{
  popu_member_t  *node = pop->head;
  int            i;

  for (i = 0; i < (nodeNum-1); i++)
    node = node->next;

  printf("Node to be deleted %d\n", i);
  printf("node fitness = %d\n", node->fitness);
  popuDelete(pop, node);

  return;
}
#endif /* 0 */

/* Function to return the best member cardinality and bit vector      */
float popuBestMember(population_t *pop, popu_member_t **member)
{
  int              i, bestIndex = -1;
  float            bestFitness = MAX_FITNESS;

  for (i = 0; i < pop->totalMembers; i++) {
    if (pop->table[i].fitness < bestFitness) {
      bestFitness = pop->table[i].fitness;
      bestIndex = i;
    }
  }

  *member = &(pop->table[bestIndex]);
  return pop->table[bestIndex].fitness;
}

/* Print the current population members - for testing                  */
void popuPrint(population_t *pop)
{
  popu_member_t     *node = NULL;
  int               i, n;

  printf("Total Number of members in the population = %d\n", pop->totalMembers);
  printf("Head fitness = %f\n", pop->table[0].fitness);
  for (n = 0; n < pop->totalMembers; n++) {
    node = &(pop->table[n]);
    printf("Fitness = %f\n", node->fitness);
    printf("Bit vector = ");
    for (i = 0; i < node->vec.members; i++) {
      printf("%d ", node->vec.vec[i]);
    }
    printf("\n");
  }

  return;
}

/* Function to print a population member */
void popuPrintMember(popu_member_t *member)
{
  int         i;

  if (member == NULL) return;

  printf("Member fitness = %f\n", member->fitness);
  printf("Bit vector = ");
  for (i = 0; i < member->vec.members; i++) {
    printf("%d ", member->vec.vec[i]);
  }
  printf("\n");
  return;
}

#if 0
/* Print the current tail of the population - for testing               */
void popuPrintTail(population_t *pop)
{
  int         i;

  if (pop->tail == NULL) return;

  printf("Tail fitness = %d\n", pop->tail->fitness);
  printf("Bit vector = ");
  for (i = 0; i < pop->tail->vec.members; i++) {
    printf("%d ", pop->tail->vec.vec[i]);
  }
  printf("\n");
  return;
}
#endif /* 0 */

