#ifndef __POPU_H
#define __POPU_H

#define MAX_INIT_POPULATION      100

typedef struct popu_member_s {
  bitvec_t               vec;
  float                  fitness;
} popu_member_t;

typedef struct population_t {
  int                 totalMembers;
  popu_member_t       table[MAX_INIT_POPULATION];
} population_t;

/* Initialize the static variables in popu.c               */
void popuInit(population_t *pop);

/* Function to add a member to the population              */
/* Returns -1 on error such as when this is a              */
/* duplicate member or memory error occurs.                */
int popuAdd(population_t *pop, bitvec_t *bvec, float fitness);

/* Function to replace the worst member of the             */
/* population with the given member.                       */
int popuReplace(population_t *pop, bitvec_t *bvec, float fitness);

#if 0
/* Function to delete a member from the population         */
/* given the pointer to the member                         */
void popuDelete(population_t *pop, popu_member_t *member);

/* Function to delete the tail of the population           */
/* This is a necessary function as we delete the worst fit */
/* member most of the time in the next generation.         */
void popuDeleteTail(population_t *pop);

/* Function to delete a certain numbered node (for testing) */
void popuDeleteNumber(population_t *pop, int nodeNum);
#endif /* 0 */

/* Function that returns the pointer to the given          */
/* member if it exists in the population;                  */
/* NULL otherwise.                                         */
popu_member_t *popuExistMember(population_t *pop, bitvec_t *bvec, 
                               float fitness);

#if 0
/* Function that returns the pointer to the population     */
/* member whose index is given.                            */
popu_member_t *popuSearchByIndex(population_t *pop, int index);
#endif /* 0 */

/* Function to return the cardinality of the best member   */
/* Returns the bit vector as an output argument            */
float popuBestMember(population_t *pop, popu_member_t **member);

/* Function to print the population                        */
void popuPrint(population_t *pop);

/* Function to print the given member of the population   */
void popuPrintMember(popu_member_t *member);

#if 0
/* Function to print the tail node details                 */
void popuPrintTail(population_t *pop);
#endif /* 0 */

#endif /* __POPU_H */
