#include <stdio.h>
#include <string.h>

#include "gen.h"
#include "set.h"

/* Initialize the set of nodes */
void initNodeSet(nodeSet *S)
{
  S->size = 0;
  memset(S->set, 0, sizeof(int) * N);

  return;
}

/* Function to determine membership of a node in a given Set */
boolean isSetMember(int nodeId, nodeSet *S)
{
  int       i;

  for (i = 0; i < S->size; i++) {
    if (nodeId == S->set[i])
      return TRUE;
  }

  return FALSE;
}

/* Add given node ID to the set of nodes */
void addNodetoNodeSet(int nodeId, nodeSet *S)
{
  if (!isSetMember(nodeId, S)) {
    S->set[S->size] = nodeId;
    (S->size)++;
  }

  return;
}

/* Function to make a copy of a set of nodes */
void copyNodeSet(nodeSet *dest, nodeSet *src)
{
  dest->size = src->size;
  memcpy(dest->set, src->set, N);

  return;
}

/* Function to return TRUE if two sets are equal; FALSE otherwise */
boolean setEqual(nodeSet *S1, nodeSet *S2)
{
  int         i, j;
  boolean     found;

  if (S1->size != S2->size) return FALSE;

  for (i = 0; i < S1->size; i++) {
    found = FALSE;
    for (j = 0; j < S2->size; j++) {
      if (S1->set[i] == S2->set[j]) {
        found = TRUE;
        break;
      }
    }
    if (!found) return FALSE;
  }

  return TRUE;
}

/* Function that returns TRUE if set S2 is a subset of set S1;        */
/* FALSE otherwise                                                    */
boolean setSubset(nodeSet *S1, nodeSet *S2)
{
  int         i, j;
  boolean     found = FALSE;

  for (i = 0; i < S2->size; i++) {
    found = FALSE;
    for (j = 0; j < S1->size; j++) {
      if (S2->set[i] == S1->set[j]) {
        found = TRUE;
        break;
      }
    }
    if (!found) return FALSE;
  }

  return(found);
}

/* Function to delete a node from a given set                    */
/* Requires moving all the remaining nodes beyond the given node */
void delNodefromNodeSet(int nodeId, nodeSet *from)
{
  int       i;

  for (i = 0; i < from->size; i++) {
    if (nodeId == from->set[i]) 
      break;
  }

  if (i != (from->size - 1)) {
    for (; i < (from->size - 1); i++)
      from->set[i] = from->set[i+1];
  }
  from->size--;

  return;
}

/* Function for union of two sets. The union is output in first argument */
void setUnion(nodeSet *V1, nodeSet *V2)
{
  int         i;

  for (i = 0; i < V2->size; i++) {
    addNodetoNodeSet(V2->set[i], V1);
  }

  return;
}

void setIntersect(nodeSet *V1, nodeSet *V2, nodeSet *out)
{
  int         i;

  for (i = 0; i < V1->size; i++) {
    if (isSetMember(V1->set[i], V2)) {
      addNodetoNodeSet(V1->set[i], out);
    }
  }

  return;
}


/* Function to return V1-V2; result is stored in V3                    */
void setDiff(nodeSet *V1, nodeSet *V2, nodeSet *V3)
{
  int     i;

  for (i = 0; i < V1->size; i++) {
    if (!isSetMember(V1->set[i], V2)) {
      addNodetoNodeSet(V1->set[i], V3);
    }
  }
}

/* V1 - V2 with the result in V1 */
void setDifference(nodeSet *V1, nodeSet *V2)
{
  int           i;
  boolean       member = TRUE;

  while (member) {
    for (i = 0; i < V1->size; i++) {
      if (isSetMember(V1->set[i], V2)) {
        delNodefromNodeSet(V1->set[i], V1);
        break;
      }
    }
    if (i == V1->size)
      member = FALSE;
  }

  return;
}


/* Function to print the nodes of a given set */
void printNodeSet(nodeSet *S)
{
  int         i;

  for (i = 0; i < S->size; i++)
    printf("%d\t", S->set[i]);
  printf("\n");

  return;
}

